import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;

/**
 * A visually laid out graph.
 */
public class VisualGraph
{
	private static final int BORDER = 20; //space to leave around the display
	private static final int NODE_SIZE = 7; //size of each node

	private ArrayList<Node> nodes;
	private ArrayList<Edge> edges;

	private int width,height;

	//make a new visual graph with the given width/height (in pixels)
	//defaults to a random graph with 20 nodes and 20% connection rate
	public VisualGraph(int width, int height)
	{
		this.width = width;
		this.height = height;

		makeRandom(20, 0.2);
	}

	/**
	 * Generate a random graph with the given nodes and connection rate
	 * @param numNodes number of nodes
	 * @param ratio edge ratio
	 */
	public void makeRandom(int numNodes, double ratio)
	{
		Random rand = new Random();

		nodes = new ArrayList<Node>();
		edges = new ArrayList<Edge>();
		
		//make the nodes
		for(int i=0; i<numNodes; i++)
		{
			nodes.add(new Node(BORDER+rand.nextInt(width-BORDER*2), BORDER+rand.nextInt(height-BORDER*2), NODE_SIZE));
		}
		//specify the edges
		for(int i=0; i<numNodes; i++)
		{
			for(int j=i+1; j<numNodes; j++)
			{
				if(rand.nextDouble() < ratio)
				{
					edges.add(new Edge(nodes.get(i), nodes.get(j)));
				}
			}
		}

	}

	/**
	 * Layout the nodes neatly. The given excluded node should not be modified (i.e., it is being modified by the user)
	 * @param exclude one node to exclude from the automatic layout; null if all nodes should be considered
	 */
	public void layout(Node exclude)
	{

		//TODO Fill this in!

	}
	
	
	/**
	 * Returns an unmodifiable list of the nodes.
	 * @return The nodes in this graph.
	 */
	public List<Node> getNodes()
	{
		return Collections.unmodifiableList(nodes); //can make unmodifiable?
	}

	/**
	 * Returns an unmodifiable list of the edges.
	 * @return The edges in this graph.
	 */
	public List<Edge> getEdges()
	{
		return Collections.unmodifiableList(edges); //can make unmodifiable?
	}

	/**
	 * Returns the width of the graph (in pixels)
	 * @return the width of the graph
	 */
	public int getWidth()
	{
		return width;
	}

	/**
	 * Returns the height of the graph (in pixels)
	 * @return the height of the graph
	 */
	public int getHeight()
	{
		return height;
	}
	
	/**
	 * An inner class to represent a node in this graph.
	 */
	public static class Node
	{
		private double x; //position of the node
		private double y;
		private int radius; //radius of the node

		public Node(double x, double y, int radius)
		{
			this.x = x;
			this.y = y;
			this.radius = radius;
		}

		public Node(int x, int y)
		{
			this(x,y,VisualGraph.NODE_SIZE);
		}
		
		//getter for the x position
		public int x()
		{
			return (int)x;
		}
		
		//getter for the y position
		public int y()
		{
			return (int)y;
		}
		
		//getter for the radius
		public int radius()
		{
			return radius;
		}
		
		//setter for the X (so others can move)
		public void setX(int x)
		{
			this.x = x;
		}
		
		//setter for the Y (so others can move)
		public void setY(int y)
		{
			this.y = y;
		}
	}

	/**
	 * An inner class representing an Edge in this graph.
	 */
	public static class Edge
	{
		private Node u; //the nodes the edge connects
		private Node v;

		public Edge(Node u, Node v)
		{
			this.u = u;
			this.v = v;
		}
		
		//the "first" node in the edge
		public Node start()
		{
			return u;
		}
		
		//the "second" node in the edge
		public Node end()
		{
			return v;
		}
	}

}
